﻿namespace U2UC.WinRT.SQLiteSample.DataAccessLayer
{
    using SQLite;
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;
    using System.Threading.Tasks;
    using U2UC.WinRT.SQLiteSample.Helpers;
    using U2UC.WinRT.SQLiteSample.Models;
    using Windows.ApplicationModel;
    using Windows.Storage;

    internal static class Dal
    {
        private static string dbPath = string.Empty;
        private static string DbPath
        {
            get
            {
                if (string.IsNullOrEmpty(dbPath))
                {
                    dbPath = Path.Combine(ApplicationData.Current.LocalFolder.Path, "Storage.sqlite");
                }

                return dbPath;
            }
        }

        public static async Task CreateDatabase()
        {
            // Create a new connection
            using (var db = new SQLiteConnection(DbPath))
            {
                // Activate Tracing
                db.Trace = true;

                // Create the table if it does not exist
                var c = db.CreateTable<Person>();
                // var info = db.GetMapping(typeof(Person));

                Person person = new Person();
                person.Id = 1;
                person.Name = "11th Doctor";
                person.Status = 0;
                person.Description = "The Eleventh Doctor was a capricious and adventurous incarnation of the renegade Time Lord known as the Doctor.";
                StorageFile file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\11th_Doctor.png"));
                person.Picture = await file.AsByteArray();
                var i = db.InsertOrReplace(person);

                person = new Person();
                person.Id = 2;
                person.Name = "Weeping Angel";
                person.Status = 2;
                person.Description = "The Weeping Angels were a species of quantum-locked humanoids, so called because their unique nature necessitated that they often covered their faces with their hands to prevent trapping each other in petrified form for eternity by looking at one another.";
                file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\Weeping_Angel.jpg"));
                person.Picture = await file.AsByteArray();
                i = db.InsertOrReplace(person);

                person = new Person();
                person.Id = 3;
                person.Name = "Silent";
                person.Status = 2;
                person.Description = "The Silents were a 'memory-proof' alien species who led a religious order known as the Silence. They seemed to be willing to do anything to kill the Doctor before he could answer the Question, including murder and kidnapping.";
                file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\Silence.jpg"));
                person.Picture = await file.AsByteArray();
                i = db.InsertOrReplace(person);

                person = new Person();
                person.Id = 4;
                person.Name = "River Song";
                person.Status = 1;
                person.Description = "'River Song', the primary alias of the third incarnation of Melody Pond, was a 'child of the TARDIS' and a known wife of the Eleventh Doctor. She was human with Time Lord characteristics, conceived by her parents, Amy Pond and Rory Williams, aboard the TARDIS as it travelled through the time vortex, and then conditioned by the Silence.";
                file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\River_Song.jpg"));
                person.Picture = await file.AsByteArray();
                i = db.InsertOrReplace(person);

                person = new Person();
                person.Id = 5;
                person.Name = "Rory Williams";
                person.Status = 0;
                person.Description = "Rory Arthur Williams — sometimes called Rory Pond — was Amy Pond's husband. He became a companion of the Eleventh Doctor on the night before their wedding, but he died and was erased from history after being absorbed by the Time Field.";
                file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\Rory.jpg"));
                person.Picture = await file.AsByteArray();
                i = db.InsertOrReplace(person);

                person = new Person();
                person.Id = 6;
                person.Name = "Dalek";
                person.Status = 2;
                person.Description = "Daleks were the armoured, mutated descendants of the Kaleds of the planet Skaro. They fought the Time Lords in the Last Great Time War, ending in the near-total destruction of both races.";
                file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\Daleks.jpg"));
                person.Picture = await file.AsByteArray();
                i = db.InsertOrReplace(person);

                person = new Person();
                person.Id = 7;
                person.Name = "Cyberman";
                person.Status = 0;
                person.Description = "Cybermen were a 'race' of cybernetically augmented humanoids. They varied greatly in design, with different factions throughout time and space.";
                file = await StorageFile.GetFileFromPathAsync(Path.Combine(Package.Current.InstalledLocation.Path, @"Assets\Pictures\Cybermen.jpg"));
                person.Picture = await file.AsByteArray();
                i = db.InsertOrReplace(person);
            }
        }

        public static void DeletePerson(Person person)
        {
            using (var db = new SQLiteConnection(DbPath))
            {
                // Activate Tracing
                db.Trace = true;

                // Object model:
                // db.Delete(person);

                // SQL Syntax:
                db.Execute("DELETE FROM Person WHERE Id = ?", person.Id);
            }
        }

        public static List<Person> GetAllPersons()
        {
            List<Person> models;

            using (var db = new SQLiteConnection(DbPath))
            {
                // Activate Tracing
                db.Trace = true;

                models = (from p in db.Table<Person>()
                          select p).ToList();
            }

            return models;
        }

        public static Person GetPersonById(int Id)
        {
            using (var db = new SQLiteConnection(DbPath))
            {
                Person m = (from p in db.Table<Person>()
                            where p.Id == Id
                            select p).FirstOrDefault();
                return m;
            }
        }

        public static void SavePerson(Person person)
        {
            using (var db = new SQLiteConnection(DbPath))
            {
                // Activate Tracing
                db.Trace = true;

                if (person.Id == 0)
                {
                    // New
                    db.Insert(person);
                }
                else
                {
                    // Update
                    db.Update(person);
                }
            }
        }
    }
}